function fp_thresh = computeNFalsePositives_PerThreshold(thresholds, scores)

%    Copyright (C) 2015  Davide Modolo
%
%    You can redistribute and/or modify this software for non-commercial use
%    under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%    For commercial use, contact the author for licensing options.
%
%    Contact: d.modolo@sms.ed.ac.uk


% given, E=#exemplars, P=#positive windows and N=#negative windows
% INPUTS:
%
%       thresholds: (P x E) - columns wise, all the possibile thresholds an 
%                             eesvm can take (#T < P). sorted descend
%       scores    : (N x E) - each cell (n, e) tells us the score of applying
%                             exemplar 'e' on negative window 'n'
%
% OUTPUT:
%       fp_thresh: matrix (P/T x E) - each cell (t, e) of tells us how many 
%                                     negative windows are retrieved (scored 
%                                     positive) by 'e' with threshold 't'.
%
% This matrix is useful because, given the indexes of the
% sorted negative windows, we can retrieve the set of negatives 
% simply as "indexes_sorted_negs(1:T(i))"

fp_thresh = zeros(size(thresholds));

% Loop over the thresholds
% start from 2 because at i=1 the thresholds produce no false positives
for i = 2:size(thresholds, 1)

    % Loop over the exemplars
    for j = 1:size(thresholds, 2)

        % since #T < P, many cells are just filled with -Inf
        if isinf(thresholds(i, j))
            if thresholds(i, j) == -Inf
                fp_thresh(i, j) = size(scores, 1);
                continue
            else
                continue
            end
        end

        % get the list of all negatives with score greater than the
        % current threshold
        x = find(scores(:, j) > thresholds(i, j));

        % just keep the last one
        fp_thresh(i, j) = x(end);

    end

end

end