function T = estimateCandidateThresholds(scores, labels)

%    Copyright (C) 2015  Davide Modolo
%
%    You can redistribute and/or modify this software for non-commercial use
%    under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
%    For commercial use, contact the author for licensing options.
%
%    Contact: d.modolo@sms.ed.ac.uk 
%
% INPUTS:
%       scores: matrix W x E, where E = number of classifiers to calibrate
%               W = number of windows in the validation set
%                        remind now eesvms scoring on their own
%       labels: window labels (1 if the window is positive, 0 otherwise)

[s_scores, i_s_scores] = sort(scores, 'descend');

labels = repmat(labels, 1, size(scores, 2));
s_labels = labels(i_s_scores);

% we have at most n_positive_windows thresholds +1 per eesvm
T = zeros(sum(labels(:,1))+1, size(scores, 2));
T(:, :) = -Inf;

% Loop over exemplars
for i = 1:size(s_labels, 2)

    % find all positives
    orig = find(s_labels(:, i) == 1);        

    % shift and subtract to the original vector
    shift = [0; orig(1:end-1)];        
    shit_diff = orig - shift;
    T_loc = find(shit_diff > 1);  % T locations

    % T_loc = 1 means that the highest scoring window is negative
    % hence, we fix the initial threshold of this eesvm to Inf
    if T_loc(1) == 1
        T(1, i) = Inf;
        T_loc(1) = [];
    end

    % get scores of the windows where the thresholds lie
    posscores = s_scores(orig(T_loc-1), i);
    negscores = s_scores(orig(T_loc-1)+1, i);

    % compute the tresholds
    T_values = mean([posscores, negscores], 2);

    % add last threshold (after the last positive), only if there is at
    % least one negative scoring lower than the positive
    if ~(orig(end)==size(scores, 1))
        T_values = [T_values; mean([s_scores(orig(end), i), s_scores(orig(end)+1, i)])];
    end

    % aggregate all the thresholds
    if T(1, i)==Inf
        T(2:size(T_values)+1, i) = T_values;
    else
        T(1:size(T_values), i) = T_values;
    end

end
end