/*    
    Copyright (C) 2015  Davide Modolo

    You can redistribute and/or modify this software for non-commercial use
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    For commercial use, contact the author for licensing options.

    Contact: d.modolo@sms.ed.ac.uk
 
 */


#include <math.h>   
#include <matrix.h>
#include <mex.h>
#include <unordered_map>
#include <vector>
#include <stdio.h>
#include <string.h>


// compile as: mex CXXFLAGS='$CXXFLAGS -std=c++11 -fPIC' fast_unique.cpp

void mexFunction(int nlhs, mxArray *out[], int nrhs, const mxArray *input[])
{
    
    // Checking number of arguments
    // Output
    if (nlhs > 1){
        mexErrMsgTxt("Error: function has only one output parameter");
        return;
    }
    
    // Inputs
    if (nrhs != 3){
        mexErrMsgTxt("Error: function needs 3 input parameters");
        return;
    }    

    //associate pointers
    uint32_t* a = (uint32_t*) mxGetPr( input[0] );
    uint32_t* b = (uint32_t*) mxGetPr( input[1] );
    int aSize = mxGetNumberOfElements(input[0]); 
    int bSize = mxGetNumberOfElements(input[1]);
        
    // number of elements
    uint32_t* c = (uint32_t*) mxGetPr( input[2] );
    uint32_t maxValue = c[0];
    
    // initialize array to store the elements to worse case scenario 
    // (all elements of A and B are unique)
    int32_t elements_added[aSize + bSize];
    
    // initialize boolean indicator to max possible value for A and B
    std::vector<bool> elements_added_bool(maxValue, false);
    
    int count = 0;
    
    // Loop over first array
    for(int i=0; i<aSize; i++)
    {
        if (!elements_added_bool[a[i]])
        {
            elements_added_bool[a[i]] = true;
            elements_added[count++] = a[i];            
        }
    }
    
    // Loop over second array
    for(int i=0; i<bSize; i++)
    {
        if (!elements_added_bool[b[i]])
        {
            elements_added_bool[b[i]] = true;
            elements_added[count++] = b[i];            
        }
    }
    
    // initialize output
    int outdims[2] = {count, 1};
    out[0] = mxCreateNumericArray(2, outdims, mxINT32_CLASS, mxREAL);
    uint32_t* unique_elements = (uint32_t*) mxGetPr(out[0]);
    
    // copy
    memcpy((void *)unique_elements, (void *)elements_added, sizeof(uint32_t)*count);
    
    return;

}
